// Include standard libraries.
#include <cstdlib>
#include <iostream>
#include <cmath>
#include <ctime>

// Include MillerRabin header.
#include "MillerRabin.h"

bool isPrime(int p) {
	if( p%2 == 0 ) {
		return false;
	}

	double pSqrt = sqrt(p);

	for(size_t i = 3; i <= pSqrt; i++) {
		if( p%i == 0 ) {
			return false;
		}
	}

	return true;
}

size_t getRandom(size_t offset, size_t end) {
	return rand() % end + offset;
}

int main(int argc, char* argv[]) {
	// Check the number of given parameters.
	if( argc != 3) {
		std::cout << "Invalid number of parameters." << std::endl;
		std::cout << "Usage: ./prime <offset> <end>" << std::endl;
		return 1;
	}

	// If enough parameters given, get offset ...
	size_t offset = atoi(argv[1]);

	// ... and end of intervall.
	size_t end = atoi(argv[1]);

	// Init random generator ...
	srand(time(NULL));

	// ... and MillerRabin test.
	MillerRabin test;

	// Loop until prime detected.
	while(true) {
		// Generate a random number, ...
		unsigned long random = getRandom(offset, end);

		// ... check if it is prime, ...
		if( test.execute(random) ) {
			// ... dump it ...
			std::cout << "Miller-Rabin: " << random << " is prime!" << std::endl;
			std::cout << "Result of Miller-Rabin is " << (isPrime(random) ?  "valid." : "not valid.") << std::endl;

			// ... and exit if it is.
			return 0;
		}
	}

	return 0;
}
