#ifndef MILLER_RABIN_CPP
#define MILLER_RABIN_CPP

// Include header file.
#include "MillerRabin.h"

unsigned long MillerRabin::modExponent(unsigned long base, unsigned long power, unsigned long modulus) {
	unsigned long result = 1;

	for(size_t i = 31; i >= 0; i--) {
		
		result = (result*result) % modulus;
		if (power & (1 << i)) {
			result = (result*base) % modulus;
		}
	}

	return result;
}

bool MillerRabin::millerRabinPass(unsigned long a, unsigned long n) {
	unsigned long aToPower, s, d, i;

	s = 0;
	d = n - 1;
	while ((d % 2) == 0) {
		 d /= 2;
		 s++;
	}

	aToPower = modExponent(a, d, n);
	if (aToPower == 1) {
		return true;
	}

	for(i=0; i < s-1; i++) {
		if (aToPower == n-1) {
			return true;
		}

		aToPower = modExponent(aToPower, 2, n);
	}

	if(aToPower == n-1) {
		return true;
	}

	return false;
}

bool MillerRabin::execute(unsigned long n) {
	if (n <= 1) {
		return false;
	}

	if (n == 2) {
		return true;
	}

	if (millerRabinPass(2, n) == true &&
		(n <= 7  || millerRabinPass( 7, n) == true) &&
		(n <= 61 || millerRabinPass(61, n) == true)) {
		return true;
	}

	else {
        return false;
	}
}

#endif
